#!/usr/bin/env python
"""

This module provides functions for moving reference files to and from the CDBS
production machine.

:Author: Christopher Hanley
:Reference: TIR-CDBS-2008-02

"""

from __future__ import division

import os,sys
import pyfits
import time


# Required for sftp support
import base64
import getpass
import socket
import traceback
import paramiko
import helper

from deliverypathconfig import WORKINGDIRROOT, DELIVERYDIR

# == Step 1 ==
#def transfer_files(username,password,hostname,hostpath,targetpath,filename,hostport=22,log=None):
    #"""
    #Transfer reference files to a directory in the CDBS delivery area.
    #"""
    
    ## setup logging
    #if log:
        #paramiko.util.log_to_file(log)
    
    ## get host key, if we know one
    #hostkeytype = None
    #hostkey = None
    #try:
        #host_keys = paramiko.util.load_host_keys(os.path.expanduser('~/.ssh/known_hosts'))
    #except IOError:
        #try:
            ## try ~/ssh/ too, because windows can't have a folder named ~/.ssh/
            #host_keys = paramiko.util.load_host_keys(os.path.expanduser('~/ssh/known_hosts'))
        #except IOError:
            #host_keys = {}

    #if host_keys.has_key(hostname):
        #hostkeytype = host_keys[hostname].keys()[0]
        #hostkey = host_keys[hostname][hostkeytype]

    ## now, connect and use paramiko Transport to negotiate SSH2 across the connection
    #try:
        #t = paramiko.Transport((hostname, hostport))
        #t.connect(username=username, password=password, hostkey=hostkey)
        #sftp = paramiko.SFTPClient.from_transport(t)

        #sftp.get(hostpath+'/'+filename, targetpath+'/'+filename)
        #t.close()
        
    #except Exception, e:
        #print '*** Caught exception: %s: %s' % (e.__class__, e)
        #traceback.print_exc()
        #try:
            #t.close()
        #except:
            #pass
        #sys.exit(1)

def mkdir_nested(directory):
    """
    
    Purpose
    =======
    Create the specified directory tree if it doesn't already exist.

    """

    print >>sys.stderr, "mkdir_nested:", directory
    
    # Remember the starting directory location
    thisdir = os.path.dirname(os.path.abspath(__file__))

    target = []
    for string in directory.split('/'):
        if string:
            target.append(string)
        
    for item in target:
        if item:
            try:
                if item == target[0]:
                    item = '/'+item
                os.chdir(item)
            except OSError:
                os.mkdir(item)
                os.chdir(item)
                
    # Return to the starting directory
    os.chdir(thisdir)

def create_dir_target(filename,root=WORKINGDIRROOT):
    """
    create_dir_target: function to define the target delivery directory
    given the reference file being delivered.
    
    :filename: string representing the name of reference file being delivered
    :root: Root path of target directory
    """
    
    # Determine the instrument type given the reference file
    hdulist = pyfits.open(filename)
    instrument = hdulist[0].header['INSTRUME'].strip()
    
    # Get today's date
    date = time.localtime()
    
    # Define the taget path name
    path = root+instrument+'/'+str(date[0])+'_'+str(date[1])+'_'+str(date[2])+'/'

    print >>sys.stderr, "create_dir_target:", filename, root, path

    return path

def copy_dir_contents(targetdir,log=None):
    """
    
    Purpose
    =======
    Copy the contents of the current directory to the specified target directory
    
    """
    
    (code, out, err) = subproc.execute(["cp -rf", '*', targetdir+"/."], log=log)
    if code != 0:
        updatelog(log,err)
        raise error.CDBSError(out, err, code)

# == Step 14 ==
def update_central_storage(deliveriesdir=DELIVERYDIR,log=None):
    """
    
    Purpose
    =======
    Copy the reference files from the working directory to the 
    approprate area on Central Storage.
    
    """
    os.chdir(deliveriesdir)
    reflist = os.listdir()
    for file in reflist:
        if file.split('.')[-1] in ["fits",'fit','FITS','FIT']:
            targetdir = "/"+helper.instlettercodes[helper.getinstname(file)]+"ref"
        # Copy the FITS files from DELIVERYDIR to the appropriate location 
        (code, out, err) = subproc.execute(["cp -f", file, targetdir + "/"], log=log)


#def update_central_storage_sftp(username,password,hostname,hostpath,targetpath,filename,hostport=22,log=None):
    #"""
    
    #Transfer reference files to a directory in the CDBS file area on CentralStorage.

    #"""
    
    ## setup logging
    #if log:
        #paramiko.util.log_to_file(log)
    
    ## get host key, if we know one
    #hostkeytype = None
    #hostkey = None
    #try:
        #host_keys = paramiko.util.load_host_keys(os.path.expanduser('~/.ssh/known_hosts'))
    #except IOError:
        #try:
            ## try ~/ssh/ too, because windows can't have a folder named ~/.ssh/
            #host_keys = paramiko.util.load_host_keys(os.path.expanduser('~/ssh/known_hosts'))
        #except IOError:
            #host_keys = {}

    #if host_keys.has_key(hostname):
        #hostkeytype = host_keys[hostname].keys()[0]
        #hostkey = host_keys[hostname][hostkeytype]

    ## now, connect and use paramiko Transport to negotiate SSH2 across the connection
    #try:
        #t = paramiko.Transport((hostname, hostport))
        #t.connect(username=username, password=password, hostkey=hostkey)
        #sftp = paramiko.SFTPClient.from_transport(t)

        #sftp.put(hostpath+'/'+filename, targetpath+'/'+filename)
        #t.close()
        
    #except Exception, e:
        #print '*** Caught exception: %s: %s' % (e.__class__, e)
        #traceback.print_exc()
        #try:
            #t.close()
        #except:
            #pass
        #sys.exit(1)


